==> avemilter-0.1.tgz: avemilter-0.1/main.c <==
/*
 * Copyright (c) 2004 Damir Bikmuhametov <boco@ufanet.ru>
 * 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: main.c,v 1.1.1.1 2004/02/09 09:32:32 boco Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sysexits.h>

#include <libmilter/mfapi.h>

#include "avecomm.h"
#include "mlfiproc.h"
#include "common.h"
#include "version.h"

char * milter_socket = NULL;
char * ave_socket = "/var/run/aveserver";
char * spooldir = "/var/spool/avemilter";
int ave_timeout = 60;
int maxmsgsize = 2048;
int tempfail = 0;
int mode_discard = 0;
int mode_reject = 0;

int daemonize(void);
void usage(void);

struct smfiDesc smfilter =
{
    "AVE Milter",		/* filter name */
    SMFI_VERSION,		/* version code -- do not change */
    0,				/* flags */
    mlfi_connect,		/* connection info filter */
    NULL,			/* SMTP HELO command filter */
    mlfi_envfrom,		/* envelope sender filter */
    mlfi_envrcpt,		/* envelope recipient filter */
    mlfi_header,		/* header filter */
    mlfi_eoh,			/* end of header */
    mlfi_body,			/* body block filter */
    mlfi_eom,			/* end of message */
    mlfi_abort,			/* message aborted */
    mlfi_close			/* connection cleanup */
};

int main(int argc, char *argv[])
{
	int c;

	puts("avemilter " AVEMILTER_VERSION "\n");

	/* process command line options */

	while ((c = getopt(argc, argv, "dfrc:m:p:s:t:")) != -1) {
		switch (c) {
			case 'd':
				if (mode_reject) {
					fprintf(stderr, "Can't select \"discard\" and \"reject\" mode simultaneously\n");
					exit(EX_USAGE);
				}
				mode_discard = 1;
				break;
			case 'r':
				if (mode_discard) {
					fprintf(stderr, "Can't select \"discard\" and \"reject\" mode simultaneously\n");
					exit(EX_USAGE);
				}
				mode_reject = 1;
				break;
			case 'f':
				tempfail = 1;
				break;
			case 't':
				ave_timeout = atoi(optarg);
				break;
			case 'm':
				maxmsgsize = atoi(optarg);
				break;
			case 'p':
				if (!optarg || !*optarg) {
					fprintf(stderr, "Illegal milter communication socket: \"%s\"\n", optarg);
					exit(EX_USAGE);
				}
				if (!(milter_socket = strdup(optarg))) {
					fprintf(stderr, "Can't allocate memory for milter communication socket path\n");
					exit(EX_UNAVAILABLE);
				}
				smfi_setconn(milter_socket);
				break;
			case 'c':
				if (!optarg || !*optarg) {
					fprintf(stderr, "Illegal AVE communication socket: \"%s\"\n", optarg);
					exit(EX_USAGE);
				}
				if (!(ave_socket = strdup(optarg))) {
					fprintf(stderr, "Can't allocate memory for AVE control socket path\n");
					exit(EX_UNAVAILABLE);
				}
				break;
			case 's':
				if (!optarg || !*optarg) {
					fprintf(stderr, "Illegal avemilter spool directory: \"%s\"\n", optarg);
					exit(EX_USAGE);
				}
				if (!(spooldir = strdup(optarg))) {
					fprintf(stderr, "Can't allocate memory for avemilter spool path\n");
					exit(EX_UNAVAILABLE);
				}
				if (access(spooldir, W_OK|R_OK)) {
					fprintf(stderr, "Can't access avemilter spool directory: \"%s\"\n", spooldir);
					exit(EX_UNAVAILABLE);
				}
				break;
			default:
				usage();
				exit(EX_USAGE);
		}
	}

	if (!milter_socket) {
		fprintf(stderr, "You MUST specify milter communication socket\n", optarg);
		exit(EX_USAGE);
	}

	/* register mail filter */

	if (smfi_register(smfilter) == MI_FAILURE) {
		fprintf(stderr, "smfi_register failed\n");
		exit(EX_UNAVAILABLE);
	}

	/* go into background */

	if (daemonize()) {
		fprintf(stderr, "Can't daemonize");
		exit(EX_UNAVAILABLE);
	}

	/* do the work */

	return smfi_main();
}

/* this routine make us daemon */

int daemonize(void)
{
	int fd;

	switch (fork()) {
		case -1:
			return -1;
		case 0:
			break;
		default:
			_exit(0);
	}

	if (setsid() == -1)
		return -1;

	chdir("/");

	if ((fd = open("/dev/null", O_RDWR, 0)) != -1) {
		dup2(fd, 0);	/* stdin */
		dup2(fd, 1);	/* stdout */
		dup2(fd, 2);	/* stderr */
		if (fd > 2)
			close(fd);
	}
	return 0;
}

/* print out short help message */

void usage(void)
{
	printf(
"Usage: avemilter [options] -p <milter socket>\n"
"Where options are:\n"
"-r - reject infected messages (default: no)\n"
"-d - discard (drop) infected messages (default: no)\n"
"-f - temporary failure on any error (default: pass message)\n"
"-c <path> - path to AVE control socket (default: /var/run/aveserver)\n"
"-s <path> - avemilter spool directory (default: /var/spool/avemilter)\n"
"-m <size> - don't check messages larger than <size> kbytes (default: 2048)\n"
"-t <sec> - AVE timeout, in seconds (default: 60 sec)\n"
"-p <socket> - (MANDATORY) milter socket, as defined in sendmail documentation\n"
"\n"
"Example:\n"
"avemilter -r -m 1024 -f -c /var/run/AveCtl -p inet:3360@127.0.0.1 -s /tmp\n"
);
}

==> avemilter-0.2.tgz: avemilter-0.2/main.c <==
/*
 * Copyright (c) 2004 Damir Bikmuhametov <boco@ufanet.ru>
 * 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: main.c,v 1.2 2004/02/22 09:29:31 boco Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sysexits.h>

#include <libmilter/mfapi.h>

#include "mlfiproc.h"
#include "version.h"

char * milter_socket = NULL;
char * av_engine_socket = "/var/run/aveserver";
char * spooldir = "/var/spool/avemilter";
int av_engine_protocol = 1;	/* 0 - old protocol, 1 - new protocol */
int av_engine_timeout = 60;
int maxmsgsize = 2048;
int tempfail = 0;
int mode_discard = 0;
int mode_reject = 0;

int daemonize(void);
void usage(void);

struct smfiDesc smfilter =
{
    "AVE Milter",		/* filter name */
    SMFI_VERSION,		/* version code -- do not change */
    0,				/* flags */
    mlfi_connect,		/* connection info filter */
    NULL,			/* SMTP HELO command filter */
    mlfi_envfrom,		/* envelope sender filter */
    mlfi_envrcpt,		/* envelope recipient filter */
    mlfi_header,		/* header filter */
    mlfi_eoh,			/* end of header */
    mlfi_body,			/* body block filter */
    mlfi_eom,			/* end of message */
    mlfi_abort,			/* message aborted */
    mlfi_close			/* connection cleanup */
};

int main(int argc, char *argv[])
{
	int c;

	puts("avemilter " AVEMILTER_VERSION "\n");

	/* process command line options */

	while ((c = getopt(argc, argv, "dfrc:e:m:p:s:t:")) != -1) {
		switch (c) {
			case 'd':
				if (mode_reject) {
					fprintf(stderr, "Can't select \"discard\" and \"reject\" mode simultaneously\n");
					exit(EX_USAGE);
				}
				mode_discard = 1;
				break;
			case 'r':
				if (mode_discard) {
					fprintf(stderr, "Can't select \"discard\" and \"reject\" mode simultaneously\n");
					exit(EX_USAGE);
				}
				mode_reject = 1;
				break;
			case 'f':
				tempfail = 1;
				break;
			case 't':
				av_engine_timeout = atoi(optarg);
				break;
			case 'm':
				maxmsgsize = atoi(optarg);
				break;
			case 'e':
				av_engine_protocol = atoi(optarg);
				if (av_engine_protocol < 0 || av_engine_protocol > 1) {
					fprintf(stderr, "Illegal AV protocol number: \"%d\"\n", av_engine_protocol);
					exit(EX_USAGE);
				}
				break;
			case 'p':
				if (!optarg || !*optarg) {
					fprintf(stderr, "Illegal milter communication socket: \"%s\"\n", optarg);
					exit(EX_USAGE);
				}
				if (!(milter_socket = strdup(optarg))) {
					fprintf(stderr, "Can't allocate memory for milter communication socket path\n");
					exit(EX_UNAVAILABLE);
				}
				smfi_setconn(milter_socket);
				break;
			case 'c':
				if (!optarg || !*optarg) {
					fprintf(stderr, "Illegal AVE communication socket: \"%s\"\n", optarg);
					exit(EX_USAGE);
				}
				if (!(av_engine_socket = strdup(optarg))) {
					fprintf(stderr, "Can't allocate memory for AVE control socket path\n");
					exit(EX_UNAVAILABLE);
				}
				break;
			case 's':
				if (!optarg || !*optarg) {
					fprintf(stderr, "Illegal avemilter spool directory: \"%s\"\n", optarg);
					exit(EX_USAGE);
				}
				if (!(spooldir = strdup(optarg))) {
					fprintf(stderr, "Can't allocate memory for avemilter spool path\n");
					exit(EX_UNAVAILABLE);
				}
				if (access(spooldir, W_OK|R_OK)) {
					fprintf(stderr, "Can't access avemilter spool directory: \"%s\"\n", spooldir);
					exit(EX_UNAVAILABLE);
				}
				break;
			default:
				usage();
				exit(EX_USAGE);
		}
	}

	if (!milter_socket) {
		fprintf(stderr, "You MUST specify milter communication socket\n");
		usage();
		exit(EX_USAGE);
	}

	/* register mail filter */

	if (smfi_register(smfilter) == MI_FAILURE) {
		fprintf(stderr, "smfi_register failed\n");
		exit(EX_UNAVAILABLE);
	}

	/* go into background */

	if (daemonize()) {
		fprintf(stderr, "Can't daemonize");
		exit(EX_UNAVAILABLE);
	}

	/* do the work */

	return smfi_main();
}

/* this routine make us daemon */

int daemonize(void)
{
	int fd;

	switch (fork()) {
		case -1:
			return -1;
		case 0:
			break;
		default:
			_exit(0);
	}

	if (setsid() == -1)
		return -1;

	chdir("/");

	if ((fd = open("/dev/null", O_RDWR, 0)) != -1) {
		dup2(fd, 0);	/* stdin */
		dup2(fd, 1);	/* stdout */
		dup2(fd, 2);	/* stderr */
		if (fd > 2)
			close(fd);
	}
	return 0;
}

/* print out short help message */

void usage(void)
{
	printf(
"Usage: avemilter [options] -p <milter socket>\n"
"Where options are:\n"
"-r - reject infected messages (default: no)\n"
"-d - discard (drop) infected messages (default: no)\n"
"-f - temporary failure on any error (default: pass message)\n"
"-e - AV engine protocol: 0 - KAV < 4.5, 1 - KAV >= 4.5 (default: 1)\n"
"-c <path> - path to AV engine control socket (default: /var/run/aveserver)\n"
"-s <path> - avemilter spool directory (default: /var/spool/avemilter)\n"
"-m <size> - don't check messages larger than <size> kbytes (default: 2048)\n"
"-t <sec> - AV engine timeout, in seconds (default: 60 sec)\n"
"-p <socket> - (MANDATORY) milter socket, as defined in sendmail documentation\n"
"\n"
"Example:\n"
"avemilter -r -m 1024 -f -c /var/run/AveCtl -p inet:3360@127.0.0.1 -s /tmp\n"
);
}
